# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rnorm_truncMultCI <- function(n, ltrunc, rtrunc, m, s) {
    .Call(`_modelSelection_rnorm_truncMultCI`, n, ltrunc, rtrunc, m, s)
}

rtmvnormCI <- function(n, mu, Sigma, lower, upper, within, method) {
    .Call(`_modelSelection_rtmvnormCI`, n, mu, Sigma, lower, upper, within, method)
}

rtmvnormProdCI <- function(n, mu, Sigma, k, lower, upper, is_low_trunc, is_up_trunc, burnin) {
    .Call(`_modelSelection_rtmvnormProdCI`, n, mu, Sigma, k, lower, upper, is_low_trunc, is_up_trunc, burnin)
}

rnlpPostCI_lm <- function(niter, burnin, thinning, y, x, p, r, tau, a_phi, b_phi, prior) {
    .Call(`_modelSelection_rnlpPostCI_lm`, niter, burnin, thinning, y, x, p, r, tau, a_phi, b_phi, prior)
}

rnlpCI <- function(niter, burnin, thinning, m, V, p, r, tau, prior) {
    .Call(`_modelSelection_rnlpCI`, niter, burnin, thinning, m, V, p, r, tau, prior)
}

modelSelectionGGMC <- function(y, prCoef, prModel, samplerPars, Omegaini) {
    .Call(`_modelSelection_modelSelectionGGMC`, y, prCoef, prModel, samplerPars, Omegaini)
}

modelSelectionGGM_globalC <- function(y, prCoef, prModel, samplerPars, Omegaini) {
    .Call(`_modelSelection_modelSelectionGGM_globalC`, y, prCoef, prModel, samplerPars, Omegaini)
}

hugeglasso <- function(S, lambda, scr, verbose, cov_output) {
    .Call(`_modelSelection_hugeglasso`, S, lambda, scr, verbose, cov_output)
}

marginalLikelihoodCI <- function(Sknownphi, Ssel, Snsel, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Slogscale) {
    .Call(`_modelSelection_marginalLikelihoodCI`, Sknownphi, Ssel, Snsel, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Slogscale)
}

normalmixGibbsCI <- function(Sx, Sn, Sp, Sncomp, Sz, Smu0, Sg, Snu0, SS0, Sq, SB, Sburnin, Sverbose) {
    .Call(`_modelSelection_normalmixGibbsCI`, Sx, Sn, Sp, Sncomp, Sz, Smu0, Sg, Snu0, SS0, Sq, SB, Sburnin, Sverbose)
}

testfunctionCI <- function(A, oldcol, newcol) {
    .Call(`_modelSelection_testfunctionCI`, A, oldcol, newcol)
}

eprod_I <- function(m, S, n, power, dof) {
    .Call(`_modelSelection_eprod_I`, m, S, n, power, dof)
}

modelSelectionEnumCI <- function(Snmodels, Smodels, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Sverbose) {
    .Call(`_modelSelection_modelSelectionEnumCI`, Snmodels, Smodels, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Sverbose)
}

modelSelectionGibbsCI <- function(SpostModeini, SpostModeiniProb, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sthinning, Sburnin, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose) {
    .Call(`_modelSelection_modelSelectionGibbsCI`, SpostModeini, SpostModeiniProb, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sthinning, Sburnin, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose)
}

greedyVarSelCI <- function(Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose) {
    .Call(`_modelSelection_greedyVarSelCI`, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose)
}

pmomMarginalKI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Sr, Smethod, SB, Slogscale, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pmomMarginalKI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Sr, Smethod, SB, Slogscale, Sngroups, Snvaringroup)
}

pmomMarginalUI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Sr, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pmomMarginalUI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Sr, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup)
}

pimomMarginalKI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Smethod, SB, Slogscale, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pimomMarginalKI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Smethod, SB, Slogscale, Sngroups, Snvaringroup)
}

pimomMarginalUI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pimomMarginalUI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup)
}

bsplineCI <- function(x, degree, knots) {
    .Call(`_modelSelection_bsplineCI`, x, degree, knots)
}

